# effects.py
# Name: Alex Cappiello
# AndrewID: acappiel
# Section: H
# Term project for 15-110, spring 2011
# All code original work unless otherwise noted

import Dialogs, DrawingTools, tkSimpleDialog
from PIL import Image, ImageEnhance

################################################################################

def findStartingPixel( newDim, oldCenter, anchor ):
    #align the old and new images
    newCenter = oldCenter
    if ( anchor == "NW" ):
        pass
    elif( anchor == "N" ):
        newCenter[ 0 ] = newDim[ 0 ] / 2
    elif( anchor == "NE" ):
        newCenter[ 0 ] = newDim[ 0 ] - oldCenter[ 0 ]
    elif( anchor == "W" ):
        newCenter[ 1 ] = newDim[ 1 ] / 2
    elif( anchor == "C" ):
        newCenter = [ newDim[ 0 ] / 2, newDim[ 1 ] / 2 ]
    elif( anchor == "E" ):
        newCenter[ 0 ] = newDim[ 0 ] - oldCenter[ 0 ]
        newCenter[ 1 ] = newDim[ 1 ] / 2
    elif( anchor == "SW" ):
        newCenter[ 1 ] = newDim[ 1 ] - oldCenter[ 1 ]
    elif( anchor == "S"):
        newCenter[ 0 ] = newDim[ 0 ] / 2
        newCenter[ 1 ] = newDim[ 1 ] - oldCenter[ 1 ]
    elif( anchor == "SE" ):
        newCenter[ 0 ] = newDim[ 0 ] - oldCenter[ 0 ]
        newCenter[ 1 ] = newDim[ 1 ] - oldCenter[ 1 ]
    return newCenter

def drawNewImage( temp, bounds, oldDim, newDim, oldCenter, newCenter, canvas ):
    offset = canvas.data.drawOffset
    #change the size of the canvas
    canvas.configure( width = newDim[ 0 ], height = newDim[ 1 ],
                      scrollregion = ( offset, offset, newDim[ 0 ],
                                       newDim[ 1 ] ) )
    DrawingTools.blankImage( newDim[ 0 ], newDim[ 1 ] )
    canvas.data.redraw = True
    canvas.data.unsavedChanges = True
    #Making a new image breaks pointers. Reset!
    DrawingTools.configure()
    pixOld = temp.load()
    pixNew = canvas.data.rawImage.load()
    for row in xrange( oldDim[ 0 ] ):
        for col in xrange( oldDim[ 1 ] ):
            #check if out of bounds (for shrinking the canvas)
            if ( bounds[ 0 ][ 0 ] + row > 0 and
                 bounds[ 0 ][ 0 ] + row < newDim[ 0 ] and
                 bounds[ 0 ][ 1 ] + col > 0 and
                 bounds[ 0 ][ 1 ] + col < newDim[ 1 ] ):
                pixNew[ bounds[ 0 ][ 0 ] + row, bounds[ 0 ][ 1 ] + col ]\
                    = pixOld[ row, col ]
    del( temp )
    
        
def resizeCanvas( root ):
    canvas = root.canvas
    #prompt user, apply if successful
    dialog = Dialogs.Resize( root, canvas.data.size[ 0 ], canvas.data.size[ 1 ],
                             title = "Dimensions for resized canvas",
                             new = False )
    if ( dialog.success ):
        anchor = dialog.anchor.get()
        oldDim = canvas.data.size
        newDim = [ dialog.width, dialog.height ]
        temp = canvas.data.rawImage
        oldCenter = [ canvas.data.size[ 0 ] / 2, canvas.data.size[ 1 ] / 2 ]
        newCenter = findStartingPixel( newDim, oldCenter, anchor )
        bounds = [ [ newCenter[ 0 ] - oldDim[ 0 ] / 2,
                     newCenter[ 1 ] - oldDim[ 1 ] / 2 ],
                   [ newCenter[ 0 ] + oldDim[ 0 ] / 2,
                     newCenter[ 1 ] + oldDim[ 1 ] / 2 ] ]
        drawNewImage( temp, bounds, oldDim, newDim, oldCenter, newCenter,
                      canvas )
        
################################################################################
        
def resizeImage( root ):
    #scale image
    canvas = root.canvas
    dialog = Dialogs.Resize( root, canvas.data.size[ 0 ], canvas.data.size[ 1 ],
                             title = "New dimensions for image", resize = True )
    if ( dialog.success ):
        offset = canvas.data.drawOffset
        canvas.data.size = ( dialog.width, dialog.height )
        canvas.data.rawImage = canvas.data.rawImage.resize(
            canvas.data.size, int( dialog.resizeFilter.get() ) )
        canvas.configure( width = dialog.width, height = dialog.height,
                          scrollregion = ( offset, offset,
                                           dialog.width, dialog. height ) )
        canvas.data.redraw = True
        DrawingTools.configure()

################################################################################

def grayscale( rgbPix ):
    newValue = ( rgbPix[ 0 ] + rgbPix[ 1 ] + rgbPix[ 2 ] ) / 3
    return ( newValue, newValue, newValue )
    
def sepia( rgbPix ):
    # conversion factors: http://www.techrepublic.com/blog/howdoi/
    # how-do-i-convert-images-to-grayscale-and-sepia-tone-using-c/120
    rgbNew = list( rgbPix )
    constRR = .393; constRG = .769; constRB = .189
    constGR = .349; constGG = .686; constGB = .168
    constBR = .272; constBG = .534; constBB = .131
    rgbNew[ 0 ] = int( rgbPix[0]*constRR+rgbPix[1]*constRG+rgbPix[2]*constRB )
    rgbNew[ 1 ] = int( rgbPix[0]*constGR+rgbPix[1]*constGG+rgbPix[2]*constGB )
    rgbNew[ 2 ] = int( rgbPix[0]*constBR+rgbPix[1]*constBG+rgbPix[2]*constBB ) 
    return tuple( rgbNew )

def brightnessInit():
    #prompt user
    global brightnessVal
    prompt = "Enter a value between 0 and 150. 75 is the base."
    brightnessVal = tkSimpleDialog.askinteger( "Brightness", prompt = prompt,
                                               initialvalue = 75,
                                               minvalue = 0, maxvalue = 150 )
    brightnessVal = 1.0 * ( brightnessVal + 50 ) / 100
    DrawingTools.pixelMod( "brightness" )

def brightness( rgbPix ):
    #change intensity of each value
    rgbNew = [ 0, 0, 0 ]
    rgbNew[ 0 ] = int( min( brightnessVal * rgbPix[ 0 ], 255 ) )
    rgbNew[ 1 ] = int( min( brightnessVal * rgbPix[ 1 ], 255 ) )
    rgbNew[ 2 ] = int( min( brightnessVal * rgbPix[ 2 ], 255 ) )
    return tuple( rgbNew )

def contrast( canvas ):
    #prompt user and set
    #PIL functions
    prompt = "Enter a value between 0 and 200. 100 is the base."
    contrastVal = tkSimpleDialog.askinteger( "Contrast", prompt = prompt,
                                               initialvalue = 100,
                                               minvalue = 0, maxvalue = 200 )
    contrastVal = 1.0 * contrastVal / 100
    enhancer = ImageEnhance.Contrast( canvas.data.rawImage )
    canvas.data.rawImage = enhancer.enhance( contrastVal )
    canvas.data.redraw = True
    DrawingTools.configure()

def color( canvas ):
    prompt = "Enter a value between 0 and 200. 100 is the base."
    colorVal = tkSimpleDialog.askinteger( "Color", prompt = prompt,
                                               initialvalue = 100,
                                               minvalue = 0, maxvalue = 200 )
    colorVal = 1.0 * colorVal / 100
    enhancer = ImageEnhance.Color( canvas.data.rawImage )
    canvas.data.rawImage = enhancer.enhance( colorVal )
    canvas.data.redraw = True
    DrawingTools.configure()

def sharpness( canvas ):
    prompt = "Enter a value between 0 and 200. 100 is the base."
    sharpVal = tkSimpleDialog.askinteger( "Sharpness", prompt = prompt,
                                               initialvalue = 100,
                                               minvalue = 0, maxvalue = 200 )
    sharpVal = 1.0 * sharpVal / 100
    enhancer = ImageEnhance.Sharpness( canvas.data.rawImage )
    canvas.data.rawImage = enhancer.enhance( sharpVal )
    canvas.data.redraw = True
    DrawingTools.configure()
